#include <libpki/pki.h>

#define BOLD     "\x1B[1m"
#define NORM     "\x1B[0m"
#define BLACK    "\x1B[30m"
#define RED      "\x1B[31m"
#define GREEN    "\x1B[32m"
#define BLUE     "\x1B[34m"

#define BG       "\x1B[47m"
#define BG_BOLD  "\x1B[31;47m"
#define BG_NORM  "\x1B[30;47m"
#define BG_RED   "\x1B[31;47m"
#define BG_GREEN "\x1B[32;47m"
#define BG_BLUE  "\x1B[34;47m"

char banner[] = 
	"\n   " BOLD "PKI CRL Management Tool " NORM "(pki-crl)\n"
	"   (c) 2008-2011 by " BOLD "Massimiliano Pala" NORM
			" and " BOLD "Open" RED "CA" NORM BOLD " Labs\n" NORM
	"       " BOLD BLUE "Open" RED "CA" NORM " Licensed software\n\n";

char usage_str[] =
	"     " BG "                                                " NORM "\n"
	"     " BG_BOLD "  USAGE: " BG_NORM "pki-crl "
		BG_GREEN "[options]                     " NORM "\n"
	"     " BG "                                                " NORM "\n\n";

void usage ( void ) {

	printf( banner );
	printf( usage_str );

	printf("   Where options are:\n");
	printf(BLUE "    -in " RED "<URI> " NORM "...........: Loads an existing CRL from <URI>\n");
	printf(BLUE "    -out " RED "<URI> " NORM "..........: Output is sent to <URI>\n");
	printf(BLUE "    -new "    "      " NORM "...........: Generates a new CRL\n");
	printf(BLUE "    -revoke " RED "<ser>" NORM ".......: Adds serial <ser> to the list of revoked certs\n");
	printf(BLUE "    -token " RED "<URI>" NORM ".........: URI of the Token to load\n");
	printf(BLUE "    -profileuri " RED "<URI>" NORM ".........: URI of a CRL profile to be loaded\n");
	printf(BLUE "    -profile " RED "<opt>" NORM ".........: Name of the CRL profile to use\n");
	printf(BLUE "    -outform " RED "<opt>" NORM ".......: Output format (PEM, DER, TXT, XML)\n");
	printf(BLUE "    -verbose"        NORM " ............: Be verbose during operations\n");
	printf(BLUE "    -debug"        NORM " ..............: Print debug information\n");
	printf(BLUE "    -help"        NORM " ...............: Print this message\n");
	printf("\n");

	exit (1);
}

int main (int argc, char *argv[] ) {
	PKI_X509_CRL *crl = NULL;
	PKI_TOKEN *tk = NULL;
	URL *inUrl = NULL;
	URL *outUrl = NULL;
	int i, error;

	int debug = 0;
	int verbose = 0;
	int log_debug = 0;
	int log_level = PKI_LOG_ERR;

	int outform = PKI_DATA_FORMAT_PEM;

	char *token = NULL;
	char *infile = NULL;
	char *outfile = "stdout";
	char *outform_s = NULL;
	char *config = NULL;

	int verify = 0;
	int verify_chain = 0;
	int new = 0;

	PKI_init_all();

	error = 0;
	for(i=1; i < argc; i++ ) {
		if( strncmp_nocase( argv[i], "-in", 3 ) == 0 ) {
			if( argv[i+1] == NULL ) {
				error=1;
				break;
			}
			infile=(argv[++i]);
		} else if(strncmp_nocase(argv[i],"-new", 4) == 0 ) {
			new=1;
		} else if ( strncmp_nocase ( argv[i], "-config", 6 ) == 0) {
			if( argv[i+1] == NULL ) {
				error=1;
				break;
			}
			config=(argv[++i]);
		} else if ( strncmp_nocase ( argv[i], "-token", 6 ) == 0) {
			if( argv[i+1] == NULL ) {
				error=1;
				break;
			}
			token=(argv[++i]);
		} else if(strncmp_nocase( argv[i], "-outform", 8) == 0 ) {
			if( argv[i+1] == NULL ) {
				error = 1;
				break;
			}
			outform_s=(argv[++i]);
		} else if ( strncmp_nocase ( argv[i], "-out", 4 ) == 0) {
			if( argv[i+1] == NULL ) {
				error=1;
				break;
			}
			outfile=(argv[++i]);
		} else if ( strcmp ( argv[i], "-outform") == 0 ) {
			if( argv[i+1] == NULL ) {
				error=1;
				break;
			}
			outform_s=argv[++i];
		} else if(strncmp_nocase(argv[i], "-h", 2) == 0 ) {
			usage();
		} else if(strncmp_nocase(argv[i],"-debug", 6) == 0 ) {
			debug=1;
		} else if(strncmp_nocase(argv[i],"-verbose", 8) == 0 ) {
			verbose=1;
		} else {
			if(verbose) fprintf(stderr, banner );
			fprintf( stderr, BOLD RED "\n    ERROR: "
				NORM "Unreckognized parameter \'" BOLD BLUE
						"%s" NORM "\'\n\n",
				argv[i]);
			exit(1);
		}
	}

	if((outUrl=getParsedUrl(outfile)) == NULL ) {
		if(verbose) fprintf(stderr, banner );
		fprintf(stderr, BOLD RED "\n    ERROR: " 
			NORM "url \"" BLUE "%s" NORM 
			"\" is not valid!\n\n", outfile );
		exit(1);
	};

	if((inUrl=getParsedUrl(infile)) == NULL ) {
		if(verbose) fprintf(stderr, banner );
		fprintf(stderr, BOLD RED "\n    ERROR: " 
			NORM "url \"" BLUE "%s" NORM 
			"\" is not valid!\n\n", infile );
		exit(1);
	};

	if(outform_s) {
		if(strncmp_nocase(outform_s, "PEM", 3) == 0 ) {
			outform = PKI_DATA_FORMAT_PEM;
		} else if (strncmp_nocase(outform_s, "DER", 3) == 0 ) {
			outform = PKI_DATA_FORMAT_ASN1;
		} else if (strncmp_nocase(outform_s, "TXT", 3) == 0 ) {
			outform = PKI_DATA_FORMAT_TXT;
		} else if (strncmp_nocase(outform_s, "XML", 3) == 0 ) {
			outform = PKI_DATA_FORMAT_XML;
		} else if (strncmp_nocase(outform_s, "B64", 3) == 0 ) {
			outform = PKI_DATA_FORMAT_B64;
		} else if (strncmp_nocase(outform_s, "URL", 3) == 0 ) {
			outform = PKI_DATA_FORMAT_URL;
		} else {
			fprintf(stderr, banner );
			fprintf(stderr, BOLD RED "\n    ERROR: " 
				NORM "output format \"" BLUE "%s" NORM 
				"\" is not valid (use one of PEM, DER, TXT, XML)!\n\n", outform_s );
			exit(1);
		};	
	};

	if( error == 1 ) {
		usage();
	}

	if( verbose ) log_level = PKI_LOG_INFO;
	if( debug ) log_debug |= PKI_LOG_FLAGS_ENABLE_DEBUG;

	if(( PKI_log_init (PKI_LOG_TYPE_STDERR, log_level, NULL,
                        log_debug, NULL )) == PKI_ERR ) {
		printf("ERROR, can not initialize logging facility!\n\n");
		exit(1);
	}

	if(verbose) fprintf( stderr, banner );

	if(verbose) {
		fprintf( stderr, BOLD "\n    Initialization:" NORM "\n");
		fprintf( stderr, 
			"    * Parameters parsed ......" GREEN " Ok" 
								NORM "\n");
		fprintf( stderr, 
			"    * Loading Certificate ...." );
	}

	if( new ) {
	} else {
		if((crl = PKI_X509_CRL_get_url( inUrl, NULL, NULL )) == NULL) {
			if(!verbose) fprintf(stderr,"\n");
			fprintf( stderr, BOLD RED "    ERROR: " 
					NORM "Can not open certificate URL " BLUE "(%s)\n\n" NORM,
						inUrl->url_s );
			exit(1);
		};
	};

	if( outfile ) {
		if( verbose ) {
			fprintf( stderr, GREEN " Ok" NORM "\n");
			fprintf( stderr, 
				"    * Saving CRL ....." );
		};

		if( PKI_X509_CERT_put(crl, outform, outfile, 
				NULL, NULL, NULL) != PKI_OK ) {
			fprintf( stderr, BOLD RED "    ERROR: " 
					NORM "Can not store CRL to URL " BLUE "(%s)\n\n" NORM,
						inUrl->url_s );
			exit(1);
		};

		if( verbose ) {
			fprintf( stderr, GREEN " Ok" NORM "\n\n");
		};
	};

	if( crl ) PKI_X509_CRL_free( crl );

	return(0);
}


